#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
Created on 2016年3月3日
@author: zeb
'''

# import modules
import ogr, os, sys, psycopg2
from PIL import Image,ImageDraw

def drawPolygon(geom, draw, w, h, minlng, maxlng, minlat, maxlat):
    for j in range(0, geom.GetGeometryCount()):
        item = geom.GetGeometryRef(j)
        for i in range(0, item.GetPointCount()-2):
            p1 = item.GetPoint(i)
            p1=(w*(p1[0]-minlng)/(maxlng-minlng),h-h*(p1[1]-minlat)/(maxlat-minlat))
            p2 = item.GetPoint(i + 1)
            p2=(w*(p2[0]-minlng)/(maxlng-minlng),h-h*(p2[1]-minlat)/(maxlat-minlat))
            draw.line(p1 + p2, fill=128)
        p1 = item.GetPoint(item.GetPointCount()-1)
        p1=(w*(p1[0]-minlng)/(maxlng-minlng),h-h*(p1[1]-minlat)/(maxlat-minlat))
        p2 = item.GetPoint(0)
        p2=(w*(p2[0]-minlng)/(maxlng-minlng),h-h*(p2[1]-minlat)/(maxlat-minlat))
        draw.line(p1 + p2, fill=128)
    return

#set server info
hosti='localhost'
porti=5432
useri='postgres'
pwdi='postgres'
dbi='mydb'

# set the working directory
os.chdir('/home/zeb/CHN_adm')

#connect the server
conn = psycopg2.connect(host=hosti, port=porti, user=useri, password=pwdi,database=dbi)
cur = conn.cursor()

#import shapefile
#execute some sqls
cur.execute('DROP TABLE IF EXISTS public.chn_adm3;')
# get the shapefile driver
driver = ogr.GetDriverByName('ESRI Shapefile')
#create table
cur.execute('CREATE TABLE public.CHN_adm3 (gid int4 NOT NULL primary key, name0 varchar(50), name1 varchar(50), name2 varchar(50), name3 varchar(50), geom public.geometry) ;')
#open shp file
shapefile = driver.Open('CHN_adm3.shp') 
if shapefile is None:
    print ('Could not open file')
    sys.exit(1)
layer = shapefile.GetLayer()
for i in range(layer.GetFeatureCount()):  
    feature = layer.GetFeature(i)
    name0 = feature.GetField("NAME_0")#.decode("utf-8")
    name1 = feature.GetField("NAME_1")#.decode("utf-8")
    name2 = feature.GetField("NAME_2")#.decode("utf-8")  
    name3 = feature.GetField("NAME_3")#.decode("utf-8")
    fid =i
    wkt = feature.GetGeometryRef().ExportToWkt()
    cur.execute("INSERT INTO public.CHN_adm3  (gid, name0, name1, name2, name3, geom) " +"VALUES (%s, %s, %s, %s, %s, ST_GeometryFromText(%s, " +"4326));", (fid, name0, name1, name2, name3, wkt))
conn.commit()
print('Shapefile import successfully!')

#export shapefile
#create the shp file
if os.path.exists('Output.shp'):
    driver.DeleteDataSource('Output.shp')
outDs = driver.CreateDataSource('Output.shp')
if outDs is None:
    print ('Could not create Output.shp')
    sys.exit(1)
outLayer = outDs.CreateLayer('output', geom_type=ogr.wkbMultiPolygon)
fid = ogr.FieldDefn( "gid", ogr.OFTInteger )
outLayer.CreateField(fid);
name0 = ogr.FieldDefn( "name0", ogr.OFTString )
name0.SetWidth( 30 )
outLayer.CreateField(name0);
name1 = ogr.FieldDefn( "name1", ogr.OFTString )
name1.SetWidth( 30 )
outLayer.CreateField(name1);
name2 = ogr.FieldDefn( "name2", ogr.OFTString )
name2.SetWidth( 30 )
outLayer.CreateField(name2);
name3 = ogr.FieldDefn( "name3", ogr.OFTString )
name3.SetWidth( 30 )
outLayer.CreateField(name3);
outDefn = outLayer.GetLayerDefn()
#fill the data into the file
cur.execute('select gid, name0, name1, name2, name3, ST_AsText(geom) AS geom from public.CHN_adm3;')
shp=cur.fetchall()
for row in shp:
    # create a new feature
    outFeature = ogr.Feature(outDefn)
    # set the geometry and attributes
    fid=row[0]
    name0=row[1]
    name1=row[2]
    name2=row[3]
    name3=row[4]
    geo = ogr.CreateGeometryFromWkt(row[5])
    outFeature.SetGeometry(geo)
    outFeature.SetField('gid', fid)
    outFeature.SetField('name0',name0)
    outFeature.SetField('name1',name1)
    outFeature.SetField('name2',name2)
    outFeature.SetField('name3',name3)
    # save the feature to the output
    outLayer.CreateFeature(outFeature)
    outFeature.Destroy()
print('export shapefile successfully!')
 
#draw shp on an image
#create the image
cur.execute("SELECT ST_XMin(bbox.geom), ST_XMax(bbox.geom), ST_YMin(bbox.geom), ST_YMax(bbox.geom) FROM (SELECT ST_Envelope(ST_Union(geom)) AS geom FROM public.CHN_adm3) AS bbox")
bbox = cur.fetchone()
minlng = bbox[0]
maxlng = bbox[1]
minlat = bbox[2]
maxlat = bbox[3]
w = 1280
h = int(w * (maxlat - minlat) / (maxlng - minlng))
img = Image.new("RGB", (w, h), "white")
draw = ImageDraw.Draw(img)
#draw geometry
cur.execute('select gid, name0, name1, name2, name3, ST_AsText(geom) AS geom from public.CHN_adm3;')
shp=cur.fetchall()
for row in shp:
    # print row[1]
    ob = ogr.CreateGeometryFromWkt(row[5])    
    gtype=ob.GetGeometryName()
    #print(gtype)
    if gtype=='MULTIPOLYGON':
        for j in range(0, ob.GetGeometryCount()):
                item = ob.GetGeometryRef(j)
                drawPolygon(item, draw,w, h, minlng, maxlng, minlat, maxlat)
    else:
        drawPolygon(ob, draw,w, h, minlng, maxlng, minlat, maxlat)
#save image
img.save('China.jpg', 'JPEG')
print('Draw shp successfully!')
cur.close()
conn.close()


#===============================================================================
# # get the attributes in this layer
# layerDefn = layer.GetLayerDefn()
# for i in range(layerDefn.GetFieldCount()):
#     fieldName=layerDefn.GetFieldDefn(i).GetNameRef()
#     fieldType=layerDefn.GetFieldDefn(i).GetType()
#===============================================================================



#if __name__=='__main__':
    

